﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml.Serialization;

namespace RetroCalendar {

    /// <summary>
    /// storage for calendars
    /// </summary>
    [Serializable]
    public class Calendar : IDisposable {

        public string StartTime;
        public int HourFormat;
        public int TimeInterval;
        public int EarlyRing;
        public bool Sound;

        [XmlArray]
        public List<CalEvent> Events;
        [XmlArray]
        public List<CalDayPad> Pads;

        [XmlIgnore]
        public bool Changed = false;

        public Calendar() {

            HourFormat = 12;
            StartTime = "7:00 AM";
            TimeInterval = 60;
            EarlyRing = 0;
            Sound = true;

            Events = new List<CalEvent>();
            Pads = new List<CalDayPad>();
        }

        public void Dispose() {

            // dereference lists
            for (int i = 0; i < Events.Count(); i++ ) {
                Events[i] = null;
            }
            Events = null;
            for (int i = 0; i < Pads.Count(); i++) {
                Pads[i] = null;
            }
            Pads = null;
        }

        /// <summary>
        /// fetch an event with the specified time
        /// </summary>
        /// <param name="cal"></param>
        /// <param name="time"></param>
        /// <returns></returns>
        public CalEvent FetchEvent(DateTime time) {

            CalEvent result = null;

            // truncate seconds
            DateTime search = new DateTime(time.Year, time.Month, time.Day, time.Hour, time.Minute, 0);

            foreach (CalEvent ev in Events) {
                if (ev.Time == search) {
                    result = ev;
                    break;
                }
            }

            return result;
        }

        /// <summary>
        /// return all events for a specifed date
        /// </summary>
        /// <param name="day"></param>
        /// <returns></returns>
        public IList<CalEvent> FetchEventsForDay(DateTime day) {

            IList<CalEvent> result = new List<CalEvent>();

            foreach (CalEvent ev in Events) {
                if (ev.Time.Date == day.Date) {
                    result.Add(ev);
                }
            }

            result = result.OrderBy(x => x.Time).ToList<CalEvent>();

            return result;
        }

        /// <summary>
        /// return all events within a specified range
        /// </summary>
        /// <param name="day"></param>
        /// <returns></returns>
        public IList<CalEvent> FetchEventsForRange(DateTime start, DateTime end) {

            IList<CalEvent> result = new List<CalEvent>();

            foreach (CalEvent ev in Events) {
                if (ev.Time.Date >= start.Date && ev.Time.Date <= end.Date) {
                    result.Add(ev);
                }
            }

            result = result.OrderBy(x => x.Time).ToList<CalEvent>();

            return result;
        }

        /// <summary>
        /// return true if a pad or event exists on the specified date
        /// </summary>
        /// <param name="day"></param>
        /// <returns></returns>
        public bool DayHasEventOrPad(DateTime day) {

            bool result = false;

            foreach (CalEvent ev in Events) {
                if (ev.Time.Date == day.Date && (ev.Content != "" || ev.Alarm)) {
                    result = true;
                    break;
                }
            }

            if (!result) {
                CalDayPad pad = FetchDayPad(day);
                if (pad != null && pad.Content != "") {
                    result = true;
                }
            }

            return result;
        }

        /// <summary>
        /// fetch a pad from the calendar with the specified date
        /// </summary>
        /// <param name="day"></param>
        /// <returns></returns>
        public CalDayPad FetchDayPad(DateTime day) {

            CalDayPad result = null;

            foreach (CalDayPad pad in Pads) {
                if (pad.Date.Date == day.Date) {
                    result = pad;
                    break;
                }
            }

            return result;
        }

        /// <summary>
        /// add an empty event to the calendar
        /// </summary>
        /// <param name="time"></param>
        /// <returns></returns>
        public CalEvent AddEvent(DateTime time) {

            // truncate seconds
            DateTime add = new DateTime(time.Year, time.Month, time.Day, time.Hour, time.Minute, 0);

            // return existing event if found
            CalEvent ev = FetchEvent(add);

            if (ev == null) {
                ev = new CalEvent();
                ev.Time = add;
                ev.Alarm = false;
                ev.Special = false;
                ev.Content = "";
                Events.Add(ev);
            }

            return ev;
        }

        /// <summary>
        /// add an empty pad to the calendar
        /// </summary>
        /// <param name="day"></param>
        /// <returns></returns>
        public CalDayPad AddDayPad(DateTime day) {

            // return existing pad if found
            CalDayPad pad = FetchDayPad(day);

            if (pad == null) {
                pad = new CalDayPad();
                pad.Date = day.Date;
                pad.Content = "";
                Pads.Add(pad);
            }

            return pad;
        }

        /// <summary>
        /// delete events within a specified date range
        /// </summary>
        /// <param name="from"></param>
        /// <param name="to"></param>
        public void DeleteEvents(DateTime from, DateTime to) {

            foreach (CalEvent ev in Events.ToArray()) {
                if (ev.Time.Date >= from.Date && ev.Time.Date <= to.Date) {
                    Events.Remove(ev);
                }
            }
        }

        /// <summary>
        /// delete the specified event
        /// </summary>
        /// <param name="ev"></param>
        public void DeleteEvent(CalEvent ev) {

            Events.Remove(ev);
        }

        /// <summary>
        /// serialize the calendar to the specified path
        /// </summary>
        /// <param name="path"></param>
        public void Save(string path) {

            // remove empty events
            foreach (CalEvent ev in Events.ToArray()) {
                ev.Content = ev.Content.TrimEnd();
                if ((ev.Alarm == false) && (ev.Special == false) && (ev.Content == "")) {
                    Events.Remove(ev);
                }
            }

            // order events by date/time
            Events = Events.OrderBy(x => x.Time).ToList();

            // remove empty pads
            foreach (CalDayPad pad in Pads.ToArray()) {
                pad.Content = pad.Content.TrimEnd();
                if (pad.Content == "") {
                    Pads.Remove(pad);
                }
            }

            // order pads by date/time
            Pads = Pads.OrderBy(x => x.Date).ToList();

            // serialize
            using (FileStream fs = new FileStream(path, FileMode.Create)) {
                XmlSerializer x = new XmlSerializer(this.GetType());
                x.Serialize(fs, this);
            }
        }

        /// <summary>
        /// unserialize and return the calendar from the specified path
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static Calendar Load(string path) {

            if (File.Exists(path)) {
                using (FileStream fs = new FileStream(path, FileMode.Open)) {
                    XmlSerializer x = new XmlSerializer(typeof(Calendar));
                    return (Calendar)x.Deserialize(fs);
                }
            }
            else {
                return null;
            }
        }


        public override string ToString() {

            StringBuilder sb = new StringBuilder();

            sb.Append("Calendar");
            sb.Append(Environment.NewLine);
            sb.Append(Environment.NewLine);
            sb.Append("StartTime: ");
            sb.Append(StartTime);
            sb.Append(Environment.NewLine);
            sb.Append("HourFormat: ");
            sb.Append(HourFormat);
            sb.Append(Environment.NewLine);
            sb.Append("TimeInterval: ");
            sb.Append(TimeInterval);
            sb.Append(Environment.NewLine);
            sb.Append("EarlyRing: ");
            sb.Append(EarlyRing);
            sb.Append(Environment.NewLine);
            sb.Append("Sound: ");
            sb.Append(Sound.ToString());
            sb.Append(Environment.NewLine);

            sb.Append(Environment.NewLine);
            sb.Append("Changed: ");
            sb.Append(Changed.ToString());
            sb.Append(Environment.NewLine);

            sb.Append(Environment.NewLine);
            sb.Append("Events");
            sb.Append(Environment.NewLine);
            foreach (CalEvent ev in Events) {
                sb.Append(ev.ToString());
            }

            sb.Append(Environment.NewLine);
            sb.Append("Pads");
            sb.Append(Environment.NewLine);
            foreach (CalDayPad pad in Pads) {
                sb.Append(pad.ToString());
            }

            return sb.ToString();
        }
    }
}
