﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace RetroCalendar.Controls {

    public partial class CalView : UserControl {

        /// <summary>
        /// current calendar
        /// </summary>
        private Calendar _cal;
        public Calendar CurrentCalendar {
            get { return _cal; }
            set {
                _cal = value;
                if (_mv != null) {
                    _mv.CurrentCalendar = _cal;
                }
                if (_dv != null) {
                    _dv.CurrentCalendar = _cal;
                }

                UpdateDate();
            }
        }

        /// <summary>
        /// current date
        /// </summary>
        private DateTime _cd;
        public DateTime CurrentDate {
            get { return _cd; }
            set {
                _cd = value;
                UpdateDate();
                UpdateView();
            }
        }

        /// <summary>
        /// views
        /// </summary>
        private DayView _dv;
        private MonthView _mv;

        /// <summary>
        /// current view
        /// </summary>
        public enum CurrentViews { Day, Month };
        public CurrentViews CurrentView = CurrentViews.Day;

        /// <summary>
        /// contents of the day pad container
        /// </summary>
        public string DayPad {
            get { return this.txtDayPad.Text; }
            set { this.txtDayPad.Text = value; }
        }

        private DateTime? _lastalarm = null;

        public CalView() {

            InitializeComponent();

            CurrentDate = DateTime.Now;
        }

        private void CalView_Load(object sender, EventArgs e) {

            CurrentDate = DateTime.Now;

            UpdateTime();
            UpdateDate();

            timer1_Tick(this, EventArgs.Empty);
            timer1.Interval = 1000;
            timer1.Enabled = true;

            _dv = new DayView();
            _dv.Width = this.panelView.Width;
            _dv.Height = this.panelView.Height;
            _dv.CurrentDate = CurrentDate;

            _mv = new MonthView();
            _mv.Width = this.panelView.Width;
            _mv.Height = this.panelView.Height;
            _mv.CurrentDate = CurrentDate;
            _mv.DateChanged += _mv_DateChanged;
        }

        /// <summary>
        /// the date was changed on the month view
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void _mv_DateChanged(object sender, EventArgs e) {

            // update private value, so won't get into an event loop
            _cd = _mv.CurrentDate;
            UpdateDate();
        }

        /// <summary>
        /// scroll to the prev or next date
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void scrollDate_Scroll(object sender, ScrollEventArgs e) {

            DateTime dd = CurrentDate;

            if (e.Type == ScrollEventType.SmallDecrement) {
                if (CurrentView == CurrentViews.Day) {
                    dd = dd.AddDays(-1);
                }
                else {
                    dd = dd.AddMonths(-1);
                }
            }
            if (e.Type == ScrollEventType.SmallIncrement) {
                if (CurrentView == CurrentViews.Day) {
                    dd = dd.AddDays(1);
                }
                else {
                    dd = dd.AddMonths(1);
                }
            }

            if (dd != CurrentDate) {
                CurrentDate = dd;
            }
        }

        /// <summary>
        /// tell control to commit any outstanding changes
        /// </summary>
        public void Commit() {

            _dv.Commit();
        }

        public void PasteText(string text) {

            if (this.txtDayPad.Focused || CurrentView == CurrentViews.Month) {
                if (this.txtDayPad.SelectedText.Length > 0) {
                    this.txtDayPad.SelectedText = text;
                }
                else {
                    this.txtDayPad.Text = this.txtDayPad.Text + text;
                }
            }
            else {
                if (CurrentView == CurrentViews.Day) {
                    _dv.PasteText(text);
                }
            }
        }

        public string SelectedText() {

            string result = "";
            if (this.CurrentView == CurrentViews.Day) {
                if (this.txtDayPad.SelectedText.Length > 0) {
                    result = this.txtDayPad.SelectedText;
                }
                else {
                    result = _dv.SelectedText();
                }
            }

            return result;
        }

        public void CopyText() {
            if (this.CurrentView == CurrentViews.Day) {
                if (this.txtDayPad.SelectedText.Length > 0) {
                    Clipboard.SetText(this.txtDayPad.SelectedText);
                    this.txtDayPad.SelectionLength = 0;
                }
                else {
                    _dv.CopyText();
                }
            }
        }

        public void CutText() {
            if (this.CurrentView == CurrentViews.Day) {
                if (this.txtDayPad.SelectedText.Length > 0) {
                    Clipboard.SetText(this.txtDayPad.SelectedText);
                    this.txtDayPad.SelectedText = "";
                }
                else {
                    _dv.CutText();
                }
            }
        }

        /// <summary>
        /// update the date
        /// </summary>
        private void UpdateDate() {

            this.dateTimePicker.Value = CurrentDate;
        }

        /// <summary>
        /// update the views with the current date
        /// </summary>
        public void UpdateView() {

            if (_cal != null) {
                if (_dv != null) {
                    _dv.CurrentDate = CurrentDate;
                }
                if (_mv != null) {
                    _mv.CurrentDate = CurrentDate;
                }
                LoadDayPad();
            }
        }

        /// <summary>
        /// update time display
        /// </summary>
        private void UpdateTime() {

            this.lblClock.Text = DateTime.Now.ToString("h:mm tt");
        }

        /// <summary>
        /// timer tick
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1_Tick(object sender, EventArgs e) {

            DateTime now = DateTime.Now;

            if (now.Second == 0) {

                UpdateTime();

                if (_cal != null) {
                    DateTime alarm = now.AddMinutes(_cal.EarlyRing);
                    CalEvent ev = _cal.FetchEvent(alarm);
                    if (ev != null) {
                        if (ev.Alarm && (_lastalarm == null || (_lastalarm.HasValue && ev.Time != _lastalarm.Value))) {
                            _lastalarm = ev.Time;
                            if (_cal.Sound) {
                                System.Threading.ThreadPool.QueueUserWorkItem(ignoredState => {
                                    using (System.IO.Stream stream = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("RetroCalendar.electronic_alarm_clock.wav")) {
                                        using (System.Media.SoundPlayer player = new System.Media.SoundPlayer(stream)) {
                                            player.PlaySync();
                                        }
                                    }
                                });
                            }
                            string msg = "Alarm at: " + ev.Time.ToLongDateString() + " " + ev.Time.ToShortTimeString() + Environment.NewLine + Environment.NewLine + ev.Content;
                            this.Invoke(new Action(() => { MessageBox.Show(this, msg, "RetroCalendar Alarm"); }));
                        }
                    }
                }
            }
            
        }

        /// <summary>
        /// text changed - save it
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtDayPad_TextChanged(object sender, EventArgs e) {

            // mark calendar updated
            _cal.Changed = true;

            // see if a daypad exists for that day - if so update it
            CalDayPad pad = _cal.FetchDayPad(CurrentDate);
            if (pad == null) {
                pad = _cal.AddDayPad(CurrentDate);
            }
            pad.Content = txtDayPad.Text;
        }

        /// <summary>
        /// set day view
        /// </summary>
        public void SetDayView() {

            if (_dv != null) {
                Panel c = this.panelView;
                c.Controls.Clear();
                c.Invalidate();

                // load day pad content - if any
                LoadDayPad();

                c.Controls.Add(_dv);

                _dv.Focus();

                CurrentView = CurrentViews.Day;
            }
        }

        /// <summary>
        /// set month view
        /// </summary>
        public void SetMonthView() {

            if (_mv != null) {
                Panel c = this.panelView;
                c.Controls.Clear();
                c.Invalidate();

                // load day pad content - if any
                LoadDayPad();

                c.Controls.Add(_mv);

                _mv.Focus();

                CurrentView = CurrentViews.Month;
            }
        }

        /// <summary>
        /// load day pad content
        /// </summary>
        private void LoadDayPad() {

            this.txtDayPad.TextChanged -= txtDayPad_TextChanged;

            this.txtDayPad.Text = "";
            CalDayPad pad = _cal.FetchDayPad(CurrentDate);
            if (pad != null) {
                this.txtDayPad.Text = pad.Content;
            }

            this.txtDayPad.TextChanged +=txtDayPad_TextChanged;
        }

        /// <summary>
        /// user picked another date
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dateTimePicker_ValueChanged(object sender, EventArgs e) {

            CurrentDate = this.dateTimePicker.Value;
        }

    }
}
