﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Reflection;

namespace RetroCalendar.Controls {

    public partial class DayView : UserControl {

        private Calendar _cal;
        public Calendar CurrentCalendar {
            get { return _cal; }
            set {
                _cal = value;
                DateChanged();
            }
        }

        private DateTime _cd;
        public DateTime CurrentDate {
            get { return _cd; }
            set {
                _cd = value;
                DateChanged();
            }
        }

        private List<DayViewRow> _rows = new List<DayViewRow>();
        private BindingSource _bs = new BindingSource();

        private Image _alarm_icon;

        private bool _enterFired = false;

        private int _focusrow = -1;

        public DayView() {

            InitializeComponent();

            _cd = DateTime.Now;
        }

        private void DayView_Load(object sender, EventArgs e) {

            _alarm_icon = Bitmap.FromStream(Assembly.GetExecutingAssembly().GetManifestResourceStream("RetroCalendar.alarm_18x18.png"));

            this.dgvEvents.Dock = DockStyle.Fill;
            this.dgvEvents.AllowUserToAddRows = false;
            this.dgvEvents.AllowUserToDeleteRows = false;
            this.dgvEvents.AllowUserToOrderColumns = false;
            this.dgvEvents.AllowUserToResizeColumns = false;
            this.dgvEvents.AllowUserToResizeRows = false;

            this.dgvEvents.MultiSelect = false;
            this.dgvEvents.SelectionMode = DataGridViewSelectionMode.CellSelect;
            this.dgvEvents.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.None;
            this.dgvEvents.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.None;
            this.dgvEvents.AutoGenerateColumns = false;

            this.dgvEvents.Font = new Font("Arial", 12, FontStyle.Bold);

            DataGridViewImageColumn icol = new DataGridViewImageColumn();
            icol.Name = "Alarm";
            icol.DataPropertyName = "Alarm";
            icol.ReadOnly = true;
            icol.Width = 20;
            icol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            icol.DefaultCellStyle.NullValue = null;
            this.dgvEvents.Columns.Add(icol);
            
            DataGridViewTextBoxColumn tcol = new DataGridViewTextBoxColumn();
            tcol.Name = "Time";
            tcol.DataPropertyName = "TimeStamp";
            tcol.ReadOnly = true;
            tcol.Width = 100;
            tcol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            SetupTimeFormat(tcol);
            this.dgvEvents.Columns.Add(tcol);

            DataGridViewTextBoxColumn ccol = new DataGridViewTextBoxColumn();
            ccol.Name = "Content";
            ccol.DataPropertyName = "Content";
            ccol.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            ccol.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
            this.dgvEvents.Columns.Add(ccol);

            this.dgvEvents.RowHeadersVisible = false;
            this.dgvEvents.ColumnHeadersVisible = false;
            this.dgvEvents.CellBorderStyle = DataGridViewCellBorderStyle.None;

            this.dgvEvents.DefaultCellStyle.SelectionBackColor = Color.White;
            this.dgvEvents.DefaultCellStyle.SelectionForeColor = Color.Black;

            this.dgvEvents.StandardTab = true;

            this.dgvEvents.DataSource = _bs;
        }

        private void DateChanged() {

            RefreshView();
        }

        /// <summary>
        /// set the time format
        /// </summary>
        /// <param name="col"></param>
        public void SetupTimeFormat(DataGridViewColumn col) {

            if (_cal.HourFormat == 12) {
                col.DefaultCellStyle.Format = "h:mm tt";
            }
            else {
                col.DefaultCellStyle.Format = "hh:mm";
            }
        }

        /// <summary>
        /// refresh the view
        /// </summary>
        public void RefreshView() {

            if (_cal != null) {

                Commit();

                _rows = new List<DayViewRow>();

                DataGridViewColumn col = this.dgvEvents.Columns["Time"];
                if (col != null) {
                    SetupTimeFormat(col);
                }

                DateTime time = new DateTime(_cd.Year, _cd.Month, _cd.Day, 0, 0, 0);
                DateTime end = new DateTime(_cd.Year, _cd.Month, _cd.Day, 23, 59, 59);
                DateTime focus = DateTime.Parse(_cd.Month + "/" + _cd.Day + "/" + _cd.Year + " " + _cal.StartTime);

                IList<CalEvent> list = _cal.FetchEventsForDay(_cd);

                _focusrow = 0;

                while (time < end) {

                    DayViewRow row = new DayViewRow();
                    row.Alarm = false;
                    row.TimeStamp = time;
                    row.Content = "";
                    foreach (CalEvent ev in list) {
                        if (ev.Time == time) {
                            row.Content = ev.Content;
                            row.Alarm = ev.Alarm;
                            break;
                        }
                    }

                    _rows.Add(row);

                    if (time == focus) {
                        _focusrow = _rows.Count - 1;
                    }

                    // bump to next time - look for inserted times
                    DateTime nxttime = time.AddMinutes(_cal.TimeInterval);
                    foreach (CalEvent ev in list) {
                        if (ev.Time > time && ev.Time < nxttime) {
                            // inserted time found - add it
                            row = new DayViewRow();
                            row.Alarm = ev.Alarm;
                            row.TimeStamp = ev.Time;
                            row.Content = ev.Content;
                            _rows.Add(row);
                        }
                        // no more times in range
                        if (ev.Time >= nxttime) {
                            break;
                        }
                    }
                    time = nxttime;
                }

                _bs.DataSource = _rows;
                _bs.ResetBindings(false);
            }

        }

        /// <summary>
        /// commit any outstanding edits
        /// </summary>
        public void Commit() {

            this.dgvEvents.EndEdit();
        }

        public void PasteText(string text) {

            this.dgvEvents.CurrentCell.Value += text;
        }

        public string SelectedText() {

            string result = "";

            DataGridViewTextBoxEditingControl editControl = (DataGridViewTextBoxEditingControl)this.dgvEvents.EditingControl;
            if (editControl != null) {
                if (editControl.SelectedText.Length > 0) {
                    result = editControl.SelectedText;
                }
            }
            return result;
        }

        public void CopyText() {

            DataGridViewTextBoxEditingControl editControl = (DataGridViewTextBoxEditingControl)this.dgvEvents.EditingControl;
            if (editControl != null) {
                if (editControl.SelectedText.Length > 0) {
                    Clipboard.SetText(editControl.SelectedText);
                    editControl.SelectionLength = 0;
                }
            }
        }

        public void CutText() {

            DataGridViewTextBoxEditingControl editControl = (DataGridViewTextBoxEditingControl)this.dgvEvents.EditingControl;
            if (editControl != null) {
                if (editControl.SelectedText.Length > 0) {
                    Clipboard.SetText(editControl.SelectedText);
                    editControl.SelectedText = "";
                }
            }
        }

        private void dgvEvents_SelectionChanged(object sender, EventArgs e) {

            int acol_index = this.dgvEvents.Columns["Alarm"].Index;
            int tcol_index = this.dgvEvents.Columns["Time"].Index;
            int ccol_index = this.dgvEvents.Columns["Content"].Index;
            if (this.dgvEvents.CurrentCell.ColumnIndex == acol_index) return;
            if (this.dgvEvents.CurrentCell.ColumnIndex == tcol_index) {
                this.dgvEvents.CurrentCell = dgvEvents.Rows[dgvEvents.CurrentRow.Index].Cells[ccol_index];
            }
            this.dgvEvents.BeginEdit(false);
        }

        private void dgvEvents_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e) {

            int index = this.dgvEvents.Columns["Alarm"].Index;
            if (e.ColumnIndex == index) {
                if (e.Value != null) {
                    bool val = Convert.ToBoolean(e.Value);
                    if (val == false) {
                        e.Value = null;
                    }
                    else {
                        e.Value = _alarm_icon;
                    }
                }
            }
        }

        private void dgvEvents_CellClick(object sender, DataGridViewCellEventArgs e) {

            int index = this.dgvEvents.Columns["Alarm"].Index;
            if (e.ColumnIndex == index) {
                bool val = Convert.ToBoolean(this.dgvEvents.Rows[e.RowIndex].Cells[e.ColumnIndex].Value);
                this.dgvEvents.Rows[e.RowIndex].Cells[e.ColumnIndex].Value = !val;
            }
        }

        private void dgvEvents_CellValueChanged(object sender, DataGridViewCellEventArgs e) {

            int index = this.dgvEvents.Columns["Alarm"].Index;
            if (e.ColumnIndex == index) {
                if (_cal != null) {
                    _cal.Changed = true;
                    bool val = Convert.ToBoolean(this.dgvEvents.Rows[e.RowIndex].Cells[e.ColumnIndex].Value);
                    DayViewRow row = _rows[e.RowIndex];
                    CalEvent ev = _cal.FetchEvent(row.TimeStamp);
                    if (ev == null) {
                        ev = _cal.AddEvent(row.TimeStamp);
                    }
                    row.Alarm = val;
                    ev.Alarm = val;

                    int ccell_index = this.dgvEvents.Columns["Content"].Index;
                    this.dgvEvents.CurrentCell = this.dgvEvents.Rows[e.RowIndex].Cells[ccell_index];
                    this.dgvEvents.BeginEdit(false);
                }
            }
        }

        private void dgvEvents_Enter(object sender, EventArgs e) {

            FocusOnStartRow();
            _enterFired = true;
        }

        private void dgvEvents_DataBindingComplete(object sender, DataGridViewBindingCompleteEventArgs e) {

            if (_enterFired) {
                FocusOnStartRow();
            }
        }

        private void FocusOnStartRow() {

            this.dgvEvents.FirstDisplayedScrollingRowIndex = _focusrow;
            int ccol_index = this.dgvEvents.Columns["Content"].Index;
            this.dgvEvents.CurrentCell = this.dgvEvents.Rows[_focusrow].Cells[ccol_index];
            this.dgvEvents.BeginEdit(false);
        }

        private void dgvEvents_Leave(object sender, EventArgs e) {

            _enterFired = false;
        }

        private void dgvEvents_CellEndEdit(object sender, DataGridViewCellEventArgs e) {

            int ccol_index = this.dgvEvents.Columns["Content"].Index;
            if (e.ColumnIndex == ccol_index) {
                if (_cal != null) {
                    DayViewRow row = _rows[this.dgvEvents.CurrentRow.Index];
                    CalEvent ev = _cal.FetchEvent(row.TimeStamp);
                    if (ev == null) {
                        ev = _cal.AddEvent(row.TimeStamp);
                    }
                    if (ev.Content != row.Content) {
                        ev.Content = row.Content;
                        _cal.Changed = true;
                    }
                }
            }
        }

    }

    /// <summary>
    /// container for a DGV row
    /// </summary>
    public class DayViewRow {

        public DateTime TimeStamp { get; set; }
        public string Content { get; set; }
        public bool Alarm { get; set; }
    }
}
