﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace RetroCalendar.Controls {

    public partial class MonthView : UserControl {

        public event EventHandler DateChanged;

        private Calendar _cal;
        public Calendar CurrentCalendar {
            get { return _cal; }
            set {
                _cal = value;
                this.Invalidate();
            }
        }

        private DateTime _cd;
        public DateTime CurrentDate {
            get { return _cd; }
            set {
                _cd = value;
                this.Invalidate();
                if (DateChanged != null) {
                    DateChanged(this, EventArgs.Empty);
                }
            }
        }

        private List<MonthViewHotSpot> _hotspots;

        public MonthView() {

            InitializeComponent();

            // double buffer to reduce flicker
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
            this.UpdateStyles();

            _cd = DateTime.Now;
        }

        private void MonthView_Paint(object sender, PaintEventArgs e) {

            DrawMonth(e.Graphics);
        }

        /// <summary>
        /// draw month based on current date
        /// </summary>
        private void DrawMonth(Graphics graphics) {

            _hotspots = new List<MonthViewHotSpot>();

            // format title
            this.lblMonth.Text = _cd.ToString("MMMM") + " " + _cd.Year.ToString();

            // backup to first day of current month
            DateTime firstdom = new DateTime(_cd.Year, _cd.Month, 1);
            // backup to first Sunday previous to first day of current month
            DateTime dd = firstdom;
            while (dd.DayOfWeek != DayOfWeek.Sunday) dd = dd.AddDays(-1);
            DateTime first = dd;
            // goto last day of current month
            DateTime lastdom = firstdom.AddMonths(1).AddDays(-1);
            // forward to next Saturday
            dd = lastdom;
            while (dd.DayOfWeek != DayOfWeek.Saturday) dd = dd.AddDays(1);
            DateTime last = dd;

            // calculate weeks in span
            TimeSpan ts = last - first;
            int weeks = (ts.Days + 1) / 7;

            int top = this.lblMonth.Height;
            int h = this.Height - top;

            int y = top;
            int ch = (int) ((double) h / (double) weeks);
            int cw = (int) ((double) this.Width / (double) 7);
            for (int i = 0; i < weeks; i++) {
                graphics.DrawLine(Pens.Black, 0, y, this.Width, y);
                y += ch;
            }

            int x = cw;
            for (int i = 1; i < 7; i++) {
                graphics.DrawLine(Pens.Black, x, top, x, this.Height);
                x += cw;
            }

            // fill entire month array
            dd = first;
            y = top;
            Font font = new Font("Arial", 12, FontStyle.Bold);
            for (int i = 0; i < weeks; i++) {
                x = 0;
                for (int j = 0; j < 7; j++) {

                    // adjust width of rightmost cell for highlighting
                    int ccw = cw;
                    if (j == 6) {
                        ccw = this.Width - x;
                    }

                    // hilite current date
                    if (dd.Date == _cd.Date) {
                        graphics.FillRectangle(Brushes.LightSalmon, new Rectangle(x+1, y+1, ccw-1, ch-1));
                    }

                    // check for event on current date - hilite if so
                    if (_cal != null) {
                        if (_cal.DayHasEventOrPad(dd)) {
                            graphics.FillRectangle(Brushes.LightSkyBlue, new Rectangle(x + 1, y + 1, ccw - 1, 10));
                        }
                    }

                    // current month numbers in black - others in gray
                    Brush brush = Brushes.Black;
                    if (dd.Month != _cd.Month) {
                        brush = Brushes.LightGray;
                    }

                    // draw the day
                    string day = dd.Day.ToString();
                    SizeF sw = graphics.MeasureString(day, font);
                    Point p = new Point(x + (cw / 2) - ((int) (sw.Width / 2)), y + (ch / 2) - ((int) (sw.Height / 2)));
                    graphics.DrawString(day, font, brush, p);

                    _hotspots.Add(new MonthViewHotSpot { Rectangle = new Rectangle(x, y, ccw, ch), Date = dd });

                    // bump to next cell
                    x += cw;

                    // incr to next date
                    dd = dd.AddDays(1);
                }
                y += ch;
            }
        }

        protected override void OnMouseClick(MouseEventArgs e) {

            if (_hotspots != null) {
                foreach (MonthViewHotSpot hs in _hotspots) {
                    if (hs.Rectangle.Contains(e.Location)) {
                        CurrentDate = hs.Date;
                        return;
                    }
                }
            }
            base.OnMouseClick(e);
        }

        protected override bool ProcessCmdKey(ref Message msg, Keys keyData) {
 	        if (keyData == Keys.Down) {
                CurrentDate = CurrentDate.AddDays(7);
                return true;
            }
            else if (keyData == Keys.Up) {
                CurrentDate = CurrentDate.AddDays(-7);
                return true;
            }
            else if (keyData == Keys.Left) {
                CurrentDate = CurrentDate.AddDays(-1);
                return true;
            }
            else if (keyData == Keys.Right) {
                CurrentDate = CurrentDate.AddDays(1);
                return true;
            }

            return base.ProcessCmdKey(ref msg, keyData);
        }
    }

    public class MonthViewHotSpot {

        public Rectangle Rectangle;
        public DateTime Date;
    }
}



