﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using RetroCalendar.Controls;

namespace RetroCalendar {

    public partial class MainForm : Form {

        private Calendar _cal;

        private string _current_path = "";
        private string _base_title;

        private string _default_file_type = "rcl";

        private PageSettings _page_settings = new PageSettings();
        private PrinterSettings _prt_settings = null;

        // setup to open a calendar on load
        private string _open = "";
        public string OpenOnLoad {
            set { _open = value; }
        }

        public MainForm() {
            
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e) {

            _base_title = this.Text;
            this.calViewControl.CurrentDate = DateTime.Now;

            if (_open != "") {
                DoOpen(_open);
            }
            if (_cal == null) {
                DoNew();
            }

            this.calViewControl.SetDayView();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e) {

            if (!CheckForCalendarChange()) {
                e.Cancel = true;
            }
        }


        /// <summary>
        /// update the form title based on current path
        /// </summary>
        private void UpdateTitle() {

            if (_current_path != "") {
                this.Text = _base_title + " - " + Path.GetFileName(_current_path);
            }
            else {
                this.Text = _base_title + " - (Untitled)";
            }
        }

        /// <summary>
        /// create a new calendar
        /// </summary>
        private void DoNew() {

            _cal = new Calendar();
            this.calViewControl.CurrentCalendar = _cal;
            this.calViewControl.SetDayView();
            UpdateTitle();
        }

        /// <summary>
        /// load a calendar file
        /// </summary>
        /// <param name="path"></param>
        public void DoOpen(string path) {

            Calendar cal = Calendar.Load(path);
            if (cal == null) {
                MessageBox.Show("Unable to load calendar file - " + path, "Open File Error", MessageBoxButtons.OK);
                return;
            }
            else {
                _cal = cal;
                _current_path = path;
            }

            this.calViewControl.CurrentCalendar = _cal;
            this.calViewControl.SetDayView();
            UpdateTitle();
        }

        /// <summary>
        /// perform a save
        /// </summary>
        private bool DoSave() {

            bool result = false;

            if (_current_path == "") {
                result = DoSaveAs();
            }
            else {
                result = DoSaveCalendar(_current_path);
            }

            return result;
        }

        /// <summary>
        /// perform a save as
        /// </summary>
        private bool DoSaveAs() {

            bool result = false;

            SaveFileDialog dlg = new SaveFileDialog();
            dlg.Title = "Save Calendar";
            dlg.Filter = "RetroCalendar Files|*." + _default_file_type + "|All Files|*.*";
            dlg.ShowDialog();

            if (dlg.FileName != "") {
                _current_path = dlg.FileName;
                if (Path.GetExtension(_current_path) == "") {
                    if (!_current_path.EndsWith(".")) {
                        _current_path += ".";
                    }
                    _current_path += _default_file_type;
                }
                result = DoSaveCalendar(_current_path);
                UpdateTitle();
            }

            return result;
        }

        /// <summary>
        /// save the calendar - handling any errors
        /// </summary>
        /// <param name="path"></param>
        private bool DoSaveCalendar(string path) {

            bool result = false;

            try {
                _cal.Save(path);
                _cal.Changed = false;
                result = true;
            }
            catch (Exception ex) {
                MessageBox.Show("Error during Save", "Error", MessageBoxButtons.OK);
            }

            return result;
        }

        /// <summary>
        /// check for updated calendar, ask use to save, return true if pending operation can proceed, otherwise false to halt operation
        /// </summary>
        /// <returns></returns>
        private bool CheckForCalendarChange() {

            bool result = false;

            this.calViewControl.Commit();

            // calendar updated, ask to save
            if (_cal.Changed) {
                DialogResult dlgresult = MessageBox.Show("Calendar has changed.\nDo you want to save the changes?", "Calendar Changed", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
                if (dlgresult == System.Windows.Forms.DialogResult.Yes) {
                    result = DoSave();
                }
                else if (dlgresult == System.Windows.Forms.DialogResult.No) {
                    result = true;
                }
            }
            // no updates, operation can proceed
            else {
                result = true;
            }

            return result;
        }

        /////////////////////////////////////////////////////////////////////////////////////
        //
        // Menu events
        //
        /////////////////////////////////////////////////////////////////////////////////////
        private void aboutToolStripMenuItem_Click(object sender, EventArgs e) {

            AboutForm frm = new AboutForm();
            frm.ShowDialog();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e) {

            if (!CheckForCalendarChange()) return;

            DoNew();
            this.calViewControl.SetDayView();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e) {

            if (!CheckForCalendarChange()) return;

            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Title = "Save Calendar";
            dlg.Filter = "RetroCalendar Files|*." + _default_file_type + "|All Files|*.*";
            dlg.ShowDialog();
            if (dlg.FileName != "") {
                DoOpen(dlg.FileName);
            }
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e) {

            this.calViewControl.Commit();

            DoSave();
        }

        private void saveAsToolStripMenuItem_Click(object sender, EventArgs e) {

            this.calViewControl.Commit();

            DoSaveAs();
        }

        private void pageSetupToolStripMenuItem_Click(object sender, EventArgs e) {

            PageSetupDialog dlg = new PageSetupDialog();
            dlg.PageSettings = _page_settings;
            DialogResult dlgresult = dlg.ShowDialog();
            if (dlgresult == System.Windows.Forms.DialogResult.OK) {
                _page_settings = dlg.PageSettings;
            }
        }
        private void printSetupToolStripMenuItem_Click(object sender, EventArgs e) {

            PrintDialog dlg = new PrintDialog();
            DialogResult dlgresult = dlg.ShowDialog();
            if (dlgresult == System.Windows.Forms.DialogResult.OK) {
                _prt_settings = dlg.PrinterSettings;
            }
        }

        private void printToolStripMenuItem_Click(object sender, EventArgs e) {

            this.calViewControl.Commit();

            PrintForm dlg = new PrintForm();
            DialogResult dlgresult = dlg.ShowDialog();
            if (dlgresult == System.Windows.Forms.DialogResult.OK) {
                string title = "(Untitled)";
                if (_current_path != "") {
                    title = Path.GetFileName(_current_path);
                }
                EventListing el = new EventListing(title, dlg.StartDate, dlg.EndDate);
                if (_prt_settings != null) {
                    el.PrinterSettings = _prt_settings;
                }
                if (_page_settings != null) {
                    el.DefaultPageSettings = _page_settings;
                }
                el.Calendar = _cal;
                el.Print();
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e) {
            this.Close();
        }

        private void dayToolStripMenuItem_Click(object sender, EventArgs e) {
            this.calViewControl.SetDayView();
        }

        private void monthToolStripMenuItem_Click(object sender, EventArgs e) {
            this.calViewControl.SetMonthView();
        }

        private void daySettingsToolStripMenuItem_Click(object sender, EventArgs e) {

            DaySettingsForm dlg = new DaySettingsForm();
            dlg.Calendar = _cal;
            DialogResult dlgresult = dlg.ShowDialog();
            if (dlgresult == DialogResult.OK) {
                if (this.calViewControl.CurrentView == CalView.CurrentViews.Day) {
                    this.calViewControl.UpdateView();
                }
            }
        }

        private void optionsToolStripMenuItem1_Click(object sender, EventArgs e) {

            AlarmControlsForm dlg = new AlarmControlsForm();
            dlg.Calendar = _cal;
            DialogResult dlgresult = dlg.ShowDialog();
        }

        private void specialTimeToolStripMenuItem_Click(object sender, EventArgs e) {

            this.calViewControl.Commit();
            SpecialTimeForm dlg = new SpecialTimeForm();
            dlg.Calendar = _cal;
            dlg.CurrentDate = this.calViewControl.CurrentDate;
            DialogResult dlgresult = dlg.ShowDialog();
            if (dlgresult == System.Windows.Forms.DialogResult.OK) {
                this.calViewControl.UpdateView();
            }
        }

        private void removeToolStripMenuItem_Click(object sender, EventArgs e) {

            this.calViewControl.Commit();
            RemoveForm dlg = new RemoveForm();
            dlg.Calendar = _cal;
            dlg.CurrentDate = this.calViewControl.CurrentDate;
            DialogResult dlgresult = dlg.ShowDialog();
            if (dlgresult == System.Windows.Forms.DialogResult.OK) {
                this.calViewControl.UpdateView();
            }
        }

        private void editToolStripMenuItem_Click(object sender, EventArgs e) {

            this.pasteToolStripMenuItem.Enabled = false;
            this.copyToolStripMenuItem.Enabled = false;
            this.cutToolStripMenuItem.Enabled = false;

            if (Clipboard.ContainsText()) {
                this.pasteToolStripMenuItem.Enabled = true;
            }
            if (this.calViewControl.SelectedText().Length > 0) {
                this.copyToolStripMenuItem.Enabled = true;
                this.cutToolStripMenuItem.Enabled = true;
            }
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e) {

            this.calViewControl.PasteText(Clipboard.GetText());
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e) {

            if (this.calViewControl.SelectedText().Length > 0) {
                this.calViewControl.CopyText();
            }
        }

        private void cutToolStripMenuItem_Click(object sender, EventArgs e) {

            if (this.calViewControl.SelectedText().Length > 0) {
                this.calViewControl.CutText();
            }
        }


    }
}
